package app.wordexam.service;

import app.wordexam.model.Item;
import app.wordexam.model.QuestionInfo;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.io.FileUtils;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;

@Service
public class FormService {

    public Map<String, List<Item>> getAnsModel(QuestionInfo questionInfo) throws JsonProcessingException {
        String section = "ans";

        Map<String, List<Item>> ansModel = new HashMap<>();

//            System.out.println(mapper.writeValueAsString(questionInfo.getInfomation().get("ans")));
        ansModel.put(section, questionInfo.getInfomation().get("ans"));

        return ansModel;
    }

    public Map<String, List<Item>> getTitleModel(QuestionInfo questionInfo) {
        String section = "title";
        Map<String, List<Item>> titleModel = new HashMap<>();
        List<Item> itemList = questionInfo.getInfomation().get(section);
        AtomicInteger index = new AtomicInteger(1);
        AtomicReference<String> type = new AtomicReference<>();
        AtomicReference<StringBuilder> stringBuilderAtomicReference = new AtomicReference<>();
        stringBuilderAtomicReference.set(new StringBuilder());


        List<Item> titleList = new ArrayList<>();
        itemList.forEach(item -> {

            if (item.getType().equals("img")) {
                Item imgItem = new Item(index.get(), item.getType(), item.getText());
                titleList.add(imgItem);
                index.incrementAndGet();

            }

            if (item.getType().equals("txt")) {
                stringBuilderAtomicReference.get().append(item.getText());
            }

        });

        Item txtItem = new Item(0, "txt", stringBuilderAtomicReference.get().toString());
        titleList.add(txtItem);

//        titleList.forEach(item -> System.out.println("title: " + item.getText()));

        titleModel.put(section, titleList);

        return titleModel;

    }


    public Map<String, List<Item>> getChoicesModel(QuestionInfo questionInfo) {
        String section = "choices";
        Map<String, List<Item>> choicesModel = new HashMap<>();
        List<Item> itemList = questionInfo.getInfomation().get(section);

        AtomicInteger index = new AtomicInteger(0);
        AtomicReference<String> type = new AtomicReference<>();
        AtomicReference<StringBuilder> stringBuilderAtomicReference = new AtomicReference<>();

        stringBuilderAtomicReference.set(new StringBuilder());
        index.set(itemList.get(0).getIndex());
        type.set(itemList.get(0).getType());

        int optionsSize = questionInfo.getInfomation().get("choices").size();
        int lastChoiceIndex = questionInfo.getInfomation().get("choices").get(optionsSize - 1).getIndex();

        List<Item> choicesList = new ArrayList<>();
        AtomicInteger count = new AtomicInteger(0);
        itemList.forEach(item -> {


            if (index.get() != item.getIndex()) {

                Item txtItem = new Item(index.get(), type.get(), stringBuilderAtomicReference.get().toString());
                choicesList.add(txtItem);
                index.set(item.getIndex());
                stringBuilderAtomicReference.set(new StringBuilder());

            }

            if (item.getType().equals("txt")) {
                stringBuilderAtomicReference.get().append(item.getText());
            }
            if (item.getType().equals("img")) {
                Item imgItem = new Item(index.get(), item.getType(), item.getText());
                choicesList.add(imgItem);
            }
            //last option
            if (count.get() == (itemList.size() - 1)) {
//                if (index.get() == lastChoiceIndex && count.get() == (itemList.size() - 1)) {
                Item txtItem = new Item(index.get(), type.get(), stringBuilderAtomicReference.get().toString());
                choicesList.add(txtItem);
            }

            count.incrementAndGet();
        });


        choicesModel.put(section, choicesList);

        return choicesModel;
    }


    public Map<String, String> getImages(QuestionInfo questionInfo) {
        Map<String, String> imagesCollection = new HashMap<>();
        List<String> images = new ArrayList<>();
        //title part
        questionInfo.getInfomation().get("title").forEach(item -> {
            if (item.getType().equals("img")) {
                images.add(item.getText());
            }
        });
        //choices part
        questionInfo.getInfomation().get("choices").forEach(item -> {
            if (item.getType().equals("img")) {
                images.add(item.getText());
            }
        });

        //image base64 map
        imagesCollection = getImagesBase64(images);

        return imagesCollection;
    }


    public Map<String, String> getImagesBase64(List<String> images) {
        Map<String, String> imagesCollection = new HashMap<>();
        images.forEach(image -> {
            try {
                String encode = getImageBase64(image);
                imagesCollection.put(image, encode);

            } catch (IOException e) {
                e.printStackTrace();
            }

        });

        return imagesCollection;

    }


    public String getImageBase64(String filename) throws IOException {
        File file = new File(String.format("doc/%s", filename));
        String encode = "";
        byte[] bytes = FileUtils.readFileToByteArray(file);
        encode = Base64.getEncoder().encodeToString(bytes);

        return encode;
    }

    public String getQuestionModel(List<QuestionInfo> questionInfoList) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
//        String questionsmodel = "";

        List<Map<String, List<Item>>> questionModelList = new ArrayList<>();
        List<QuestionInfo> formList = new ArrayList<>();

        //title
        questionInfoList.forEach(questionInfo -> {
            Map<String, List<Item>> questionModel = new HashMap<>();
            try {
                questionModel.put("ans", getAnsModel(questionInfo).get("ans"));
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
            questionModel.put("title", getTitleModel(questionInfo).get("title"));
            questionModel.put("choices", getChoicesModel(questionInfo).get("choices"));


            questionModelList.add(questionModel);
        });

        return mapper.writeValueAsString(questionModelList);
    }

}
