package app.wordexam.service;

import app.wordexam.model.Item;
import org.jsoup.nodes.Element;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class QuestionService {

    public String getImgFolder(List<Element> paragraphs) {
        AtomicReference<String> imgFolder = new AtomicReference<>("");
        paragraphs.forEach(paragraph -> {
            paragraph.getAllElements().forEach(element -> {
                element.childNodes().forEach(node -> {
                    node.attributes().forEach(attribute -> {
                        if (attribute.getKey().equals("src")) {
                            String link = attribute.getValue();
                            if (link.length() != 0) {
                                imgFolder.set(link.split("/")[0]);

                            }
                        }

                    });
                });
            });

        });

        return imgFolder.get();
    }

    public Map<String, List<Item>> getAns(List<Element> paragraphs) {
//        ( 43)
        String section = "ans";

        AtomicInteger position = new AtomicInteger(0);
        AtomicReference<Boolean> startup = new AtomicReference<>(false);
        Map<String, List<Item>> items = new HashMap<>();

        paragraphs.forEach(paragraph -> {
            paragraph.getAllElements().forEach(element -> {
                element.childNodes().forEach(node -> {
                    node.attributes().forEach(attribute -> {

                        if (attribute.getKey().equals("#text")) {

                            String text = attribute.getValue();

                            Pattern pattern = Pattern.compile("[\\(（]");
                            Matcher matcher = pattern.matcher(text);
                            if (matcher.find()) {
                                startup.set(true);

                            }
                        }

                        if (attribute.getKey().equals("#text") && startup.get() == true) {
                            String text = attribute.getValue();
                            String regex = "(\\(.*)?(\\d)";
                            Pattern pattern = Pattern.compile(regex);
                            Matcher matcher = pattern.matcher(text);
                            if (matcher.find() == true) {
//                                System.out.println("question service ans: " + matcher.group(2));
                                Item item = new Item(position.get(), "ans", matcher.group(2));
                                List<Item> itemList = new ArrayList<>();
//                                ObjectMapper mapper = new ObjectMapper();
//                                try {
//                                    System.out.println(mapper.writeValueAsString(item));
//                                } catch (JsonProcessingException e) {
//                                    e.printStackTrace();
//                                }
                                itemList.add(item);
                                items.put(section, itemList);

                                startup.set(false);
                            }

                        }


                    });
                });
            });

        });

        return items;
    }

    public Map<String, List<Item>> getTitle(List<Element> paragraphs) {


        String section = "title";
        AtomicInteger position = new AtomicInteger(0);
        Map<String, List<Item>> items = new HashMap<>();
        List<Item> itemList = new ArrayList<>();

        AtomicReference<Boolean> startup = new AtomicReference<>(false);


        Item emptyItem = new Item(position.get(), "txt", " ");
        itemList.add(emptyItem);
        paragraphs.forEach(paragraph -> {

            String text = paragraph.text();
            Pattern pattern = Pattern.compile("[\\)）](.*)\\u2460");
            Matcher matcher = pattern.matcher(text);
            if (matcher.find()) {
                Item item = new Item(position.get(), "txt", matcher.group(1));
                itemList.add(item);
                position.incrementAndGet();
            }

        });

        paragraphs.forEach(paragraph -> {
            paragraph.getAllElements().forEach(element -> {
                element.childNodes().forEach(node -> {
                    node.attributes().forEach(attribute -> {


                        if (attribute.getKey().equals("#text")) {
                            Pattern pattern = Pattern.compile("[\\)）]");
                            Matcher matcher = pattern.matcher(attribute.getValue());
                            if (matcher.find()) {
                                startup.set(true);

                            }
                        }

                        //close title
                        if (attribute.getKey().equals("#text")) {
                            Pattern pattern = Pattern.compile("\\u2460");
                            Matcher matcher = pattern.matcher(attribute.getValue());
                            if (matcher.find()) {
                                startup.set(false);

                            }
                        }


                        if (startup.get() == true) {

                            if (attribute.getKey().equals("src")) {
                                String img = attribute.getValue();
                                Boolean isContains = itemList.stream().anyMatch(item -> item.getText().equals(img));
                                if (!isContains) {
                                    Item item = new Item(position.get(), "img", img);
                                    itemList.add(item);

                                }


                                position.incrementAndGet();
                            }


                        }


                    });
                });
            });
        });
        items.put(section, itemList);

        return items;
    }

    //①②③④
//    ①9312 ②9313③9314④9315
// 0:option1,  1:option2
    //item的position記錄第幾個選項
    public Map<String, List<Item>> getChoices(List<Element> paragraphs) {
        String section = "choices";
        AtomicInteger index = new AtomicInteger(0);
        Map<String, List<Item>> items = new HashMap<>();

        List<Item> itemList = new ArrayList<>();

        AtomicReference<Boolean> startup = new AtomicReference<>(false);

        paragraphs.forEach(element -> {
            //尋找開始點
            Pattern pattern = Pattern.compile("①(.*)②");
            Matcher matcher = pattern.matcher(element.text());
            if (matcher.find()) {
                String txt = matcher.group(1);
                Item item = new Item(index.get(), "txt", txt);
                itemList.add(item);
                index.incrementAndGet();
            }


            //option 2
            pattern = Pattern.compile(".*②(.*)③.*");
            matcher = pattern.matcher(element.text());
            if (matcher.find()) {
                Item item = new Item(index.get(), "txt", matcher.group(1));
                itemList.add(item);
                index.incrementAndGet();
            }

            //option 3
            pattern = Pattern.compile(".*③(.*)④.*");
            matcher = pattern.matcher(element.text());
            if (matcher.find()) {
                Item item = new Item(index.get(), "txt", matcher.group(1));
                itemList.add(item);
                index.incrementAndGet();
            }

            //option 4
            pattern = Pattern.compile(".*④(.*)");
            matcher = pattern.matcher(element.text());
            if (matcher.find()) {
                Item item = new Item(index.get(), "txt", matcher.group(1));
                itemList.add(item);
                index.incrementAndGet();
            }


        });



        //檢查選項是否重覆
        Set<String> itemSet = new HashSet<>();
        itemList.stream().map(item -> item.getText().toLowerCase())
                .filter(txt -> !itemSet.contains(txt))
                .forEach(itemSet::add);

        if (itemList.size() != itemSet.size()) {
            List<Item> revisedItemList = new ArrayList<>();
            index.set(1);
            itemList.forEach(item -> {
                Item rItem = new Item(item.getIndex(),item.getType(),String.format("重覆選項%s%s",index.get(),item.getText()));
                index.incrementAndGet();
                revisedItemList.add(rItem);
            });
            itemList.clear();
            itemList.addAll(revisedItemList);
        }


        //沒有選項, 自己產生四個
        if (itemList.size() == 0) {
            for (int i = 0; i < 4; i++) {
                Item item = new Item(i, "txt", String.format("選項%s", i + 1));
                itemList.add(item);
            }
        }

        items.put(section, itemList);

        return items;
    }


}
